@@ -22,10 +22,14 @@ fn find_boundary(s: &str, i: usize) -> usize {
   i
 }
 
-// This input getter runs get_event and buffers the input until a lone . appears on a line
-// Then it returns that buffer.
+// This input getter runs get_event and buffers the input with expected editing features
+// Initial contents of the buffer is given as a vector of newline terminated strings
+// A prefix can be given, which is then printed at start of every line and not included in input
+// A terminator can be given.
+// If given: input is returned after terminator has been entered alone on a line. Else on newline.
 pub fn event_input(
   state: &mut super::HighlightingUI,
+  initial_buffer: Vec<String>,
   prefix: Option<char>,
   terminator: Option<char>, // If none take only one line
 ) -> Result<Vec<String>, crossterm::ErrorKind> {
@@ -34,12 +38,18 @@ pub fn event_input(
   // Set the cursor to be visible, so our moves are visible
   stdout.queue(crossterm::cursor::Show)?;
 
-  // Create the buffer and the variables to move in it
-  let mut buffer = Vec::new();
-  buffer.push("\n".to_string());
+  // Set up buffer and variables for moving in it
+  let mut buffer = initial_buffer;
+  if buffer.len() == 0 { buffer.push("\n".to_string()); } // The buffer mustn't be empty
   let mut lindex = 0; // Line index, lin-dex
   let mut chindex = 0; // Char index, ch-index
 
+  // Variable for tracking how many steps back in history
+  // we are when moving back in history
+  let mut hoffset = state.command_history.len();
+  // And one for keeping current input while moving about in history
+  let mut semi_history = "\n".to_string();
+
   // Then the distances we need to remember between printing
   let mut dists = super::print::PrintData{ height: 0, cursor_y: 0, cursor_x: 0 };
 
@@ -167,7 +177,7 @@ pub fn event_input(
             }
             // Else, add a line
             else {
-              // Insert the newline
+              // Insert a newline to properly terminate current line
               buffer[lindex].insert(chindex, '\n');
               chindex += 1;
               // Split of the string at current index, inserting the resulting strings into buffer
@@ -176,16 +186,14 @@ pub fn event_input(
               // If the line left behind is now a lone dot on a line, delete it and return
               // Check if we just created the terminating line
               let mut iter = buffer[lindex].chars();
-              if iter.next() == terminator {
-                if iter.next() == Some('\n') {
-                  // Remove the terminating line
+              if iter.next() == terminator && iter.next() == Some('\n') {
+                // Remove the terminating line
+                buffer.remove(lindex);
+                // Check to clear unexpected line created after terminating
+                if buffer[lindex] == "\n" {
                   buffer.remove(lindex);
-                  // Check to clear unexpected line created after terminating
-                  if buffer[lindex] == "\n" {
-                    buffer.remove(lindex);
-                  }
-                  ret = true;
                 }
+                ret = true;
               }
               // Else increment and reset chindex.
               else {
@@ -226,11 +234,43 @@ pub fn event_input(
           },
   
           (KeyCode::Up, KeyModifiers::NONE) | (KeyCode::Down, KeyModifiers::NONE) => {
-            // First move to the indicated line, if possible
-            match key.code {
-              KeyCode::Up => { lindex = lindex.saturating_sub(1); },
-              KeyCode::Down => { if lindex < buffer.len() - 1 { lindex += 1; } },
-              _ => (),
+            // Go back/forth in history if in one-line mode
+            if terminator.is_none() {
+              match key.code {
+                KeyCode::Up => {
+                  // If leaving present
+                  if hoffset == state.command_history.len() {
+                    // Save current input line as semi history, unwrap or shouldn't ever be needed
+                    semi_history = buffer.pop().unwrap_or("\n".to_string());
+                  }
+                  else {
+                    buffer.pop();
+                  }
+                  // Then move into history
+                  hoffset = hoffset.saturating_sub(1);
+                },
+                KeyCode::Down => {
+                  // If not in the present, move forward in history
+                  if hoffset < state.command_history.len() { hoffset += 1; }
+                  buffer.pop();
+                },
+                _ => (),
+              }
+              // Read that history entry into the buffer
+              buffer.push(
+                state.command_history
+                  .get(hoffset) // Get history at offset
+                  .map(|line| line.clone()) // Convert from &str to String
+                  .unwrap_or(semi_history.clone()) // If none we have arrived in the present
+              )
+            }
+            else {
+              // First move to the indicated line, if possible
+              match key.code {
+                KeyCode::Up => { lindex = lindex.saturating_sub(1); },
+                KeyCode::Down => { if lindex < buffer.len() - 1 { lindex += 1; } },
+                _ => (),
+              }
             }
             // Then try to go to goal_chindex and place chindex within the new line
             match goal_chindex {
